"""Setup tests for this package."""
from collective.embeddedpage.testing import EMBEDDEDPAGE_INTEGRATION_TESTING
from Products.CMFPlone.utils import get_installer

import unittest


class TestSetup(unittest.TestCase):
    """Test that collective.embeddedpage is properly installed."""

    layer = EMBEDDEDPAGE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.installer = get_installer(self.portal, self.layer["request"])

    def test_product_installed(self):
        """Test if collective.embeddedpage is installed."""
        self.assertTrue(self.installer.is_product_installed("collective.embeddedpage"))

    def test_browserlayer(self):
        """Test that ICollectiveEmbeddedpageLayer is registered."""
        from collective.embeddedpage.interfaces import ICollectiveEmbeddedpageLayer
        from plone.browserlayer import utils

        self.assertIn(ICollectiveEmbeddedpageLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = EMBEDDEDPAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.installer = get_installer(self.portal, self.layer["request"])
        self.installer.uninstall_product("collective.embeddedpage")

    def test_product_uninstalled(self):
        """Test if collective.embeddedpage is cleanly uninstalled."""
        self.assertFalse(self.installer.is_product_installed("collective.embeddedpage"))

    def test_browserlayer_removed(self):
        """Test that ICollectiveEmbeddedpageLayer is removed."""
        from collective.embeddedpage.interfaces import ICollectiveEmbeddedpageLayer
        from plone.browserlayer import utils

        self.assertNotIn(ICollectiveEmbeddedpageLayer, utils.registered_layers())



from collective.embeddedpage.interfaces import ICollectiveEmbeddedpageLayer
from collective.embeddedpage.interfaces import IEmbeddedPage
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from plone.restapi.serializer.dxcontent import SerializeToJson
from Products.CMFPlone.utils import safe_unicode
from zope.component import adapter
from zope.interface import implementer


@implementer(ISerializeToJson)
@adapter(IEmbeddedPage, ICollectiveEmbeddedpageLayer)
class CustomSerializeToJson(SerializeToJson):
    def __call__(self, version=None, include_items=True):
        serialization = super().__call__(version=version, include_items=include_items)
        view = api.content.get_view("view", self.context, self.request)
        data = view.process_page()
        serialization.update(
            {
                "text": {
                    "data": safe_unicode(data["content"]),
                    "content-type": data["content-type"],
                    "encoding": "utf-8",
                },
            }
        )
        return serialization



from collective.embeddedpage.interfaces import IEmbeddedPage
from collective.embeddedpage.testing import EMBEDDEDPAGE_INTEGRATION_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.interfaces import IDexterityFTI
from Products.CMFPlone.utils import get_installer
from zope.component import createObject
from zope.component import queryUtility

import unittest


class EmbeddedPageIntegrationTest(unittest.TestCase):

    layer = EMBEDDEDPAGE_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.installer = get_installer(self.portal, self.layer["request"])

    def test_schema(self):
        fti = queryUtility(IDexterityFTI, name="EmbeddedPage")
        schema = fti.lookupSchema()
        self.assertEqual(IEmbeddedPage, schema)

    def test_fti(self):
        fti = queryUtility(IDexterityFTI, name="EmbeddedPage")
        self.assertTrue(fti)

    def test_factory(self):
        fti = queryUtility(IDexterityFTI, name="EmbeddedPage")
        factory = fti.factory
        obj = createObject(factory)
        self.assertTrue(IEmbeddedPage.providedBy(obj))

    def test_adding(self):
        self.portal.invokeFactory("EmbeddedPage", "EmbeddedPage")
        self.assertTrue(IEmbeddedPage.providedBy(self.portal["EmbeddedPage"]))



from lxml import etree
from Products.Five.browser import BrowserView
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib.parse import urljoin
from urllib.parse import urlparse

import chardet
import lxml
import re
import requests


class EmbeddedPageView(BrowserView):

    template = ViewPageTemplateFile("embeddedpage.pt")

    def get_zope_request_http_headers(self):
        """Return a dict with all x-*http headers in their original format"""
        headers = {}
        for name, value in self.request.environ.items():
            if not name.startswith("HTTP_"):
                continue
            name = name[5:].lower().replace("_", "-")
            headers[name] = f"{value}"
        return self.filter_x_http_headers(headers)

    def filter_x_http_headers(self, headers):
        filtered = {}
        for name, value in headers.items():
            if name.lower().startswith("x-"):
                filtered[name] = value
        return filtered

    def process_page(self):
        data = {
            "content-type": "text/html",
            "content": "",
        }

        # external js resources
        resource = self.request.form.get("embeddedpage_get_resource", "")
        if resource:
            return {
                "content-type": "application/javascript",
                "content": requests.get(resource).content,
            }

        request_type = self.request["REQUEST_METHOD"]
        method = getattr(requests, request_type.lower(), requests.get)
        params = {
            "url": self.context.url,
            # Forward request x-* headers
            "headers": self.get_zope_request_http_headers(),
        }
        if request_type == "GET":
            params["params"] = self.request.form
        else:
            params["data"] = self.request.form
            # Plone mix GET and POST parameters, and customer 'complex search' use same parameter for two things...  # noqa
            # https://docs.plone.org/develop/plone/serving/http_request_and_response.html#post-variables  # noqa
            if "komplexesuche" in params["data"].get("ifab_modus", ""):
                params["data"]["ifab_modus"] = "suchergebnis"
        try:
            response = method(**params)
        except requests.exceptions.MissingSchema:
            return data

        # Forward response x-* headers
        for k, v in self.filter_x_http_headers(response.headers).items():
            self.request.response.setHeader(k, v)

        # Normalize charset to unicode
        content = response.content
        if content.strip() == "":
            data["content"] = ""
            return data

        encoding = None
        content_type_header = response.headers.get("Content-Type", "")
        if content_type_header is not None:
            match = re.match(r"text/html; charset=(.*)", content_type_header)
            if match is not None:
                # text/html with an encoding
                encoding = match.group(1)
        if encoding is None:
            # Magic ONLY if not text/html with an encoding
            encoding = chardet.detect(content)["encoding"]

        try:
            content = content.decode(encoding)
        except Exception:
            # use default decoding on errors
            content = content.decode("utf-8")
        # https://stackoverflow.com/a/28545721/2116850
        content = re.sub(r"\<\?xml.*encoding.*\?\>\ *?\n", "", content)
        el = lxml.html.fromstring(content)
        url = self.context.absolute_url()
        for script in el.findall(".//script"):
            src = script.attrib.get("src", "")
            if src == "":
                continue
            script.attrib["src"] = f"{url}?embeddedpage_get_resource={src}"
        for iframe in el.findall(".//iframe"):
            src = iframe.attrib.get("src", "")
            if urlparse(src).scheme != "":
                continue
            iframe.attrib["src"] = urljoin(self.context.url, src)
        body = el.find("body")
        if body is not None:
            for link in el.findall(".//head//link"):
                body.insert(0, link)
            el = body
        data["content"] = etree.tostring(el, method="html")
        return data

    def __call__(self):
        data = self.process_page()
        if data.get("content-type", "text/html") != "text/html":
            self.request.response.setHeader("content-type", data["content-type"])
            self.request.response.setBody(data["content"])
            return

        self.embeddedpage = data["content"]
        return self.template()



from plone.dexterity.interfaces import IDexterityFTI
from zope.component import queryUtility


OLD_BEHAVIORS = [
    "plone.app.content.interfaces.INameFromTitle",
    "plone.app.dexterity.behaviors.discussion.IAllowDiscussion",
    "plone.app.dexterity.behaviors.exclfromnav.IExcludeFromNavigation",
    "plone.app.dexterity.behaviors.id.IShortName",
    "plone.app.contenttypes.behaviors.richtext.IRichTextBehavior",
    "plone.app.relationfield.behavior.IRelatedItems",
    "plone.app.versioningbehavior.behaviors.IVersionable",
    "plone.app.contenttypes.behaviors.tableofcontents.ITableOfContents",
    "plone.app.lockingbehavior.behaviors.ILocking",
]


def migrate_behaviors(context):
    # Migrate FTI
    fti = queryUtility(IDexterityFTI, name="EmbeddedPage")
    behavior_list = [a for a in fti.behaviors if a not in OLD_BEHAVIORS]
    fti.behaviors = behavior_list
    context.runImportStepFromProfile("collective.embeddedpage:default", "typeinfo")



from collective.embeddedpage.testing import EMBEDDEDPAGE_INTEGRATION_TESTING
from httmock import all_requests
from httmock import HTTMock
from plone import api
from plone.restapi.interfaces import ISerializeToJson
from zope.component import getMultiAdapter

import unittest


class TestCustomSerializeToJson(unittest.TestCase):

    layer = EMBEDDEDPAGE_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        with api.env.adopt_roles(["Manager"]):
            self.epage = api.content.create(
                container=self.portal,
                type="EmbeddedPage",
                id="epage",
                url="https://plone.org",
            )

    def serialize(self, obj):
        serializer = getMultiAdapter((obj, self.request), ISerializeToJson)
        return serializer()

    def test_serializer(self):
        @all_requests
        def response_link(url, request):
            return {
                "status_code": 200,
                "content": "<div>Main Page</div>",
            }

        with HTTMock(response_link):
            data = self.serialize(self.epage)
        self.assertEqual(
            {
                "data": "<div>Main Page</div>",
                "content-type": "text/html",
                "encoding": "utf-8",
            },
            data["text"],
        )



"""Installer for the collective.embeddedpage package."""

from setuptools import find_packages
from setuptools import setup

import sys


assert sys.version_info >= (
    3,
    7,
    0,
), "collective.embeddedpage 3.x.x requires Python 3.7.0+. Please downgrade to collective.embeddedpage 2.x.x for Python 2 and Plone 4.3/5.1."

long_description = "\n\n".join(
    [
        open("README.rst").read(),
        open("CONTRIBUTORS.rst").read(),
        open("CHANGES.rst").read(),
    ]
)


setup(
    name="collective.embeddedpage",
    version="3.0.3",
    description="Add-on to embed remote HTML pages into the Plone CMS",
    long_description=long_description,
    # Get more from https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 5.2",
        "Framework :: Plone :: 6.0",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Operating System :: OS Independent",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
    ],
    keywords="Python Plone",
    author="kitconcept GmbH",
    author_email="info@kitconcept.com",
    url="https://pypi.python.org/pypi/collective.embeddedpage",
    license="GPL version 2",
    packages=find_packages("src", exclude=["ez_setup"]),
    namespace_packages=["collective"],
    package_dir={"": "src"},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "chardet",
        "lxml",
        "plone.api",
        "plone.app.contenttypes",
        "plone.app.dexterity",
        "plone.app.lockingbehavior",
        "plone.app.relationfield",
        "plone.app.versioningbehavior",
        "plone.restapi",
        "Products.GenericSetup>=1.8.2",
        "requests",
        "setuptools",
        "z3c.jbot",
    ],
    extras_require={
        "test": [
            "cssselect",
            "httmock",
            "robotframework==3.1.2",
            "plone.app.testing",
            "plone.testing",
            "plone.app.contenttypes",
            "plone.app.robotframework[debug]",
            "zest.releaser[recommended]",
            "towncrier",
            "zestreleaser.towncrier",
            "wheel",
            "black",
            "isort",
            "flake8",
        ],
    },
    entry_points="""
    [z3c.autoinclude.plugin]
    target = plone
    """,
)
